@@ -2,14 +2,15 @@ import pathlib
 import shlex
 from typing import Any, Literal, Optional, Sequence, Union
 
-from ..types import StrOrBytesPath, StrOrPath
-from .core import run
+from libvcs._internal.run import run
+
+from ..types import StrOrBytesPath, StrPath
 
 _CMD = Union[StrOrBytesPath, Sequence[StrOrBytesPath]]
 
 
 class Git:
-    def __init__(self, dir: StrOrPath):
+    def __init__(self, *, dir: StrPath):
         """Lite, typed, pythonic wrapper for git(1).
 
         Parameters
@@ -35,6 +36,7 @@ class Git:
     def run(
         self,
         args: _CMD,
+        *,
         # Print-and-exit flags
         version: Optional[bool] = None,
         help: Optional[bool] = None,
@@ -42,6 +44,7 @@ class Git:
         man_path: Optional[bool] = None,
         info_path: Optional[bool] = None,
         # Normal flags
+        C: Optional[Union[StrOrBytesPath, list[StrOrBytesPath]]] = None,
         cwd: Optional[StrOrBytesPath] = None,
         git_dir: Optional[StrOrBytesPath] = None,
         work_tree: Optional[StrOrBytesPath] = None,
@@ -69,8 +72,10 @@ class Git:
 
         Parameters
         ----------
-        cwd : :attr:`libvcs.cmd.types.StrOrBytesPath`, optional
-            ``-C <path>``, Defaults to :attr:`~.cwd`
+        cwd : :attr:`libvcs.cmd.types.StrOrBytesPath`, optional, passed to subprocess's
+            ``cwd`` the command runs from. Defaults to :attr:`~.cwd`.
+        C : :attr:`libvcs.cmd.types.StrOrBytesPath`, optional
+            ``-C <path>``
         git_dir : :attr:`libvcs.cmd.types.StrOrBytesPath`, optional
             ``--git-dir <path>``
         work_tree : :attr:`libvcs.cmd.types.StrOrBytesPath`, optional
@@ -117,7 +122,7 @@ class Git:
         Examples
         --------
         >>> git = Git(dir=tmp_path)
-        >>> git.run(['help'])  # doctest: +NORMALIZE_WHITESPACE
+        >>> git.run(['help'])
         "usage: git [--version] [--help] [-C <path>]..."
         """
 
@@ -146,8 +151,11 @@ class Git:
         #
         # Flags
         #
-        if cwd is not None:
-            cli_args.append(f"-C {cwd}")
+        if C is not None:
+            if not isinstance(C, list):
+                C = [C]
+            C = [str(c) for c in C]
+            cli_args.extend(["-C", C])
         if git_dir is not None:
             cli_args.extend(["--git-dir", str(git_dir)])
         if work_tree is not None:
@@ -173,10 +181,11 @@ class Git:
         if no_optional_locks is True:
             cli_args.append("--no-optional-locks")
 
-        return run(cmd=cli_args, **kwargs)
+        return run(args=cli_args, **kwargs)
 
     def clone(
         self,
+        *,
         url: str,
         separate_git_dir: Optional[StrOrBytesPath] = None,
         template: Optional[str] = None,
@@ -241,7 +250,7 @@ class Git:
         if template is not None:
             local_flags.append(f"--template={template}")
         if separate_git_dir is not None:
-            local_flags.append(f"--separate-git-dir={separate_git_dir}")
+            local_flags.append(f"--separate-git-dir={separate_git_dir!r}")
         if (filter := kwargs.pop("filter", None)) is not None:
             local_flags.append(f"--filter={filter}")
         if depth is not None:
@@ -304,6 +313,7 @@ class Git:
 
     def fetch(
         self,
+        *,
         reftag: Optional[Any] = None,
         deepen: Optional[str] = None,
         depth: Optional[str] = None,
@@ -378,7 +388,7 @@ class Git:
         local_flags: list[str] = []
 
         if submodule_prefix is not None:
-            local_flags.append(f"--submodule-prefix={submodule_prefix}")
+            local_flags.append(f"--submodule-prefix={submodule_prefix!r}")
         if (filter := kwargs.pop("filter", None)) is not None:
             local_flags.append(f"--filter={filter}")
         if depth is not None:
@@ -461,6 +471,7 @@ class Git:
 
     def rebase(
         self,
+        *,
         upstream: Optional[str] = None,
         onto: Optional[str] = None,
         branch: Optional[str] = None,
@@ -550,7 +561,7 @@ class Git:
         if onto:
             local_flags.extend(["--onto", onto])
         if context:
-            local_flags.extend(["--C", context])
+            local_flags.extend(["--C", str(context)])
 
         if exec:
             local_flags.extend(["--exec", shlex.quote(exec)])
@@ -657,6 +668,7 @@ class Git:
 
     def pull(
         self,
+        *,
         reftag: Optional[Any] = None,
         repository: Optional[str] = None,
         deepen: Optional[str] = None,
@@ -852,7 +864,7 @@ class Git:
         # Fetch-related arguments
         #
         if submodule_prefix is not None:
-            local_flags.append(f"--submodule-prefix={submodule_prefix}")
+            local_flags.append(f"--submodule-prefix={submodule_prefix!r}")
         if (filter := kwargs.pop("filter", None)) is not None:
             local_flags.append(f"--filter={filter}")
         if depth is not None:
@@ -935,6 +947,7 @@ class Git:
 
     def init(
         self,
+        *,
         template: Optional[str] = None,
         separate_git_dir: Optional[StrOrBytesPath] = None,
         object_format: Optional[Literal["sha1", "sha256"]] = None,
@@ -994,7 +1007,7 @@ class Git:
         if template is not None:
             local_flags.append(f"--template={template}")
         if separate_git_dir is not None:
-            local_flags.append(f"--separate-git-dir={separate_git_dir}")
+            local_flags.append(f"--separate-git-dir={separate_git_dir!r}")
         if object_format is not None:
             local_flags.append(f"--object-format={object_format}")
         if branch is not None:
@@ -1011,3 +1024,627 @@ class Git:
         return self.run(
             ["init", *local_flags, "--", *required_flags], check_returncode=False
         )
+
+    def help(
+        self,
+        *,
+        all: Optional[bool] = None,
+        verbose: Optional[bool] = None,
+        no_external_commands: Optional[bool] = None,
+        no_aliases: Optional[bool] = None,
+        config: Optional[str] = None,
+        guides: Optional[str] = None,
+        info: Optional[str] = None,
+        man: Optional[str] = None,
+        web: Optional[str] = None,
+        **kwargs,
+    ):
+        """Help info. Wraps `git help <https://git-scm.com/docs/git-help>`_.
+
+        Parameters
+        ----------
+        all : bool
+            Prints everything.
+
+        no_external_commands : bool
+            For use with ``all``, excludes external commands.
+
+        no_aliases : bool
+            For use with ``all``, excludes aliases.
+
+        verbose : bool
+            For us with ``all``, on by default.
+
+        config : bool
+            List all config vars.
+
+        guides : bool
+            List concept guides.
+
+        info : bool
+            Display man page in info format.
+
+        man : bool
+            Man page.
+
+        web : bool
+            Man page in HTML.
+
+        Examples
+        --------
+        >>> git = Git(dir=tmp_path)
+
+        >>> git.help()
+        "usage: git [--version] [--help] [-C <path>]..."
+
+        >>> git.help(all=True)
+        "See 'git help <command>' to read about a specific subcommand..."
+
+        >>> git.help(info=True)
+        "usage: git [--version] [--help] [-C <path>] [-c <name>=<value>]..."
+
+        >>> git.help(man=True)
+        "usage: git [--version] [--help] [-C <path>] [-c <name>=<value>]..."
+        """
+        local_flags: list[str] = []
+
+        if verbose is True:
+            local_flags.append("--verbose")
+        if all is True:
+            local_flags.append("--all")
+        if no_external_commands is True:
+            local_flags.append("--no-external-commands")
+        if no_aliases is True:
+            local_flags.append("--no-aliases")
+        if config is True:
+            local_flags.append("--config")
+        if guides is True:
+            local_flags.append("--guides")
+        if info is True:
+            local_flags.append("--info")
+        if man is True:
+            local_flags.append("--man")
+        if web is True:
+            local_flags.append("--web")
+
+        return self.run(["help", *local_flags], check_returncode=False)
+
+    def reset(
+        self,
+        *,
+        quiet: Optional[bool] = None,
+        refresh: Optional[bool] = None,
+        no_refresh: Optional[bool] = None,
+        pathspec_from_file: Optional[StrOrBytesPath] = None,
+        pathspec: Optional[Union[StrOrBytesPath, list[StrOrBytesPath]]] = None,
+        soft: Optional[bool] = None,
+        mixed: Optional[bool] = None,
+        hard: Optional[bool] = None,
+        merge: Optional[bool] = None,
+        keep: Optional[bool] = None,
+        commit: Optional[str] = None,
+        recurse_submodules: Optional[bool] = None,
+        no_recurse_submodules: Optional[bool] = None,
+        **kwargs,
+    ):
+        """Reset HEAD. Wraps `git help <https://git-scm.com/docs/git-help>`_.
+
+        Parameters
+        ----------
+        quiet : bool
+        no_refresh : bool
+        refresh : bool
+        pathspec_from_file : :attr:`libvcs.cmd.types.StrOrBytesPath`
+        pathspec_file_nul : bool
+        pathspec : :attr:`libvcs.cmd.types.StrOrBytesPath` or list
+            :attr:`libvcs.cmd.types.StrOrBytesPath`
+        soft : bool
+        mixed : bool
+        hard : bool
+        merge : bool
+        keep : bool
+        commit : str
+
+        Examples
+        --------
+        >>> git = Git(dir=git_local_clone.dir)
+
+        >>> git.reset()
+        ''
+
+        >>> git.reset(soft=True, commit='HEAD~1')
+        ''
+        """
+        local_flags: list[str] = []
+
+        if quiet is True:
+            local_flags.append("--quiet")
+        if no_refresh is True:
+            local_flags.append("--no-refresh")
+        if refresh is True:
+            local_flags.append("--refresh")
+        if pathspec_from_file is not None:
+            local_flags.append(f"--pathspec_from_file={pathspec_from_file!r}")
+
+        # HEAD to commit form
+        if soft is True:
+            local_flags.append("--soft")
+
+        if mixed is True:
+            local_flags.append("--mixed")
+
+        if hard is True:
+            local_flags.append("--hard")
+
+        if merge is True:
+            local_flags.append("--merge")
+
+        if keep is True:
+            local_flags.append("--keep")
+
+        if commit is True:
+            local_flags.append(commit)
+
+        if recurse_submodules:
+            local_flags.append("--recurse-submodules")
+        elif no_recurse_submodules:
+            local_flags.append("--no-recurse-submodules")
+
+        if pathspec is not None:
+            if not isinstance(pathspec, list):
+                pathspec = [pathspec]
+        else:
+            pathspec = []
+
+        return self.run(
+            ["reset", *local_flags, *(["--", *pathspec] if len(pathspec) else [])],
+            check_returncode=False,
+        )
+
+    def checkout(
+        self,
+        *,
+        quiet: Optional[bool] = None,
+        progress: Optional[bool] = None,
+        no_progress: Optional[bool] = None,
+        pathspec_from_file: Optional[StrOrBytesPath] = None,
+        pathspec: Optional[Union[StrOrBytesPath, list[StrOrBytesPath]]] = None,
+        force: Optional[bool] = None,
+        ours: Optional[bool] = None,
+        theirs: Optional[bool] = None,
+        no_track: Optional[bool] = None,
+        guess: Optional[bool] = None,
+        no_guess: Optional[bool] = None,
+        _list: Optional[bool] = None,
+        detach: Optional[bool] = None,
+        merge: Optional[bool] = None,
+        ignore_skip_worktree_bits: Optional[bool] = None,
+        patch: Optional[bool] = None,
+        orphan: Optional[str] = None,
+        conflict: Optional[str] = None,
+        overwrite_ignore: Optional[bool] = None,
+        no_overwrite_ignore: Optional[bool] = None,
+        recurse_submodules: Optional[bool] = None,
+        no_recurse_submodules: Optional[bool] = None,
+        overlay: Optional[bool] = None,
+        no_overlay: Optional[bool] = None,
+        commit: Optional[str] = None,
+        branch: Optional[str] = None,
+        new_branch: Optional[str] = None,
+        start_point: Optional[str] = None,
+        treeish: Optional[str] = None,
+        **kwargs,
+    ):
+        """Switches branches or checks out files. Wraps
+        `git checkout <https://git-scm.com/docs/git-checkout>`_ (`git co`).
+
+        Parameters
+        ----------
+        quiet : bool
+        progress : bool
+        no_progress : bool
+        pathspec_from_file : :attr:`libvcs.cmd.types.StrOrBytesPath`
+        pathspec : :attr:`libvcs.cmd.types.StrOrBytesPath` or list
+            :attr:`libvcs.cmd.types.StrOrBytesPath`
+        force : bool
+        ours : bool
+        theirs : bool
+        no_track : bool
+        guess : bool
+        no_guess : bool
+        ignore_skip_worktree_bits : bool
+        merge : bool
+        _list : bool
+        detach : bool
+        patch : bool
+        orphan : bool
+        conflict : str
+        overwrite_ignore : bool
+        no_overwrite_ignore : bool
+        commit : str
+        branch : str
+        new_branch : str
+        start_point : str
+        treeish : str
+
+        Examples
+        --------
+        >>> git = Git(dir=git_local_clone.dir)
+
+        >>> git.checkout()
+        "Your branch is up to date with 'origin/master'."
+
+        >>> git.checkout(branch='origin/master', pathspec='.')
+        ''
+        """
+        local_flags: list[str] = []
+
+        if quiet is True:
+            local_flags.append("--quiet")
+        if progress is True:
+            local_flags.append("--progress")
+        elif no_progress is True:
+            local_flags.append("--no-progress")
+
+        if force is True:
+            local_flags.append("--force")
+
+        if ours is True:
+            local_flags.append("--ours")
+
+        if theirs is True:
+            local_flags.append("--theirs")
+
+        if detach is True:
+            local_flags.append("--detach")
+
+        if orphan is True:
+            local_flags.append("--orphan")
+
+        if conflict is True:
+            local_flags.append(f"--conflict={conflict}")
+
+        if commit is True:
+            local_flags.append(commit)
+
+        if branch is True:
+            local_flags.append(branch)
+
+        if new_branch is True:
+            local_flags.append(new_branch)
+
+        if start_point is True:
+            local_flags.append(start_point)
+
+        if treeish is True:
+            local_flags.append(treeish)
+
+        if recurse_submodules:
+            local_flags.append("--recurse-submodules")
+        elif no_recurse_submodules:
+            local_flags.append("--no-recurse-submodules")
+
+        if pathspec is not None:
+            if not isinstance(pathspec, list):
+                pathspec = [pathspec]
+        else:
+            pathspec = []
+
+        return self.run(
+            ["checkout", *local_flags, *(["--", *pathspec] if len(pathspec) else [])],
+            check_returncode=False,
+        )
+
+    def status(
+        self,
+        *,
+        verbose: Optional[bool] = None,
+        long: Optional[bool] = None,
+        short: Optional[bool] = None,
+        branch: Optional[bool] = None,
+        z: Optional[bool] = None,
+        column: Optional[Union[bool, str]] = None,
+        no_column: Optional[bool] = None,
+        ahead_behind: Optional[bool] = None,
+        no_ahead_behind: Optional[bool] = None,
+        renames: Optional[bool] = None,
+        no_renames: Optional[bool] = None,
+        find_renames: Optional[Union[bool, str]] = None,
+        porcelain: Optional[Union[bool, str]] = None,
+        untracked_files: Optional[Literal["no", "normal", "all"]] = None,
+        ignored: Optional[Literal["traditional", "no", "matching"]] = None,
+        ignored_submodules: Optional[Literal["untracked", "dirty", "all"]] = None,
+        pathspec: Optional[Union[StrOrBytesPath, list[StrOrBytesPath]]] = None,
+        **kwargs,
+    ):
+        """Status of working tree. Wraps
+        `git status <https://git-scm.com/docs/git-status>`_.
+
+        `git ls-files` has similar params (e.g. `z`)
+
+        Parameters
+        ----------
+        verbose : bool
+        long : bool
+        short : bool
+        branch : bool
+        z : bool
+        column : bool
+        no_column : bool
+        ahead_behind : bool
+        no_ahead_behind : bool
+        find_renames : bool
+        no_find_renames : bool
+        porcelain : str, bool
+        untracked_files : "no", "normal", "all"
+        ignored : "traditional", "no", "matching"
+        ignored_submodules : "untracked", "dirty", "all"
+        pathspec : :attr:`libvcs.cmd.types.StrOrBytesPath` or list
+            :attr:`libvcs.cmd.types.StrOrBytesPath`
+
+        Examples
+        --------
+        >>> git = Git(dir=git_local_clone.dir)
+
+        >>> git.status()
+        "On branch master..."
+
+        >>> pathlib.Path(git_local_clone.dir / 'new_file.txt').touch()
+
+        >>> git.status(porcelain=True)
+        '?? new_file.txt'
+
+        >>> git.status(porcelain='1')
+        '?? new_file.txt'
+
+        >>> git.status(porcelain='2')
+        '? new_file.txt'
+
+        >>> git.status(C=git_local_clone.dir / '.git', porcelain='2')
+        '? new_file.txt'
+        """
+        local_flags: list[str] = []
+
+        if verbose is True:
+            local_flags.append("--verbose")
+
+        if long is True:
+            local_flags.append("--long")
+
+        if short is True:
+            local_flags.append("--short")
+
+        if branch is True:
+            local_flags.append("--branch")
+
+        if z is True:
+            local_flags.append("--z")
+
+        if untracked_files is not None and isinstance(untracked_files, str):
+            local_flags.append(f"--untracked-files={untracked_files}")
+
+        if ignored is not None and isinstance(column, str):
+            local_flags.append(f"--ignored={ignored}")
+
+        if ignored_submodules is not None:
+            if isinstance(column, str):
+                local_flags.append(f"--ignored-submodules={ignored_submodules}")
+            else:
+                local_flags.append("--ignored-submodules")
+
+        if column is not None:
+            if isinstance(column, str):
+                local_flags.append(f"--column={column}")
+            else:
+                local_flags.append("--column")
+        elif no_column is not None:
+            local_flags.append("--no-column")
+
+        if porcelain is not None:
+            if isinstance(porcelain, str):
+                local_flags.append(f"--porcelain={porcelain}")
+            else:
+                local_flags.append("--porcelain")
+
+        if find_renames is True:
+            if isinstance(find_renames, str):
+                local_flags.append(f"--find-renames={find_renames}")
+            else:
+                local_flags.append("--find-renames")
+
+        if pathspec is not None:
+            if not isinstance(pathspec, list):
+                pathspec = [pathspec]
+        else:
+            pathspec = []
+
+        return self.run(
+            ["status", *local_flags, *(["--", *pathspec] if len(pathspec) else [])],
+            check_returncode=False,
+        )
+
+    def config(
+        self,
+        *,
+        replace_all: Optional[bool] = None,
+        get: Optional[str] = None,
+        get_all: Optional[bool] = None,
+        get_regexp: Optional[str] = None,
+        get_urlmatch: Optional[tuple[str, str]] = None,
+        system: Optional[bool] = None,
+        local: Optional[bool] = None,
+        worktree: Optional[bool] = None,
+        file: Optional[StrOrBytesPath] = None,
+        blob: Optional[str] = None,
+        remove_section: Optional[bool] = None,
+        rename_section: Optional[bool] = None,
+        unset: Optional[bool] = None,
+        unset_all: Optional[bool] = None,
+        _list: Optional[bool] = None,
+        fixed_value: Optional[bool] = None,
+        no_type: Optional[bool] = None,
+        null: Optional[bool] = None,
+        name_only: Optional[bool] = None,
+        show_origin: Optional[bool] = None,
+        show_scope: Optional[bool] = None,
+        get_color: Optional[Union[str, bool]] = None,
+        get_colorbool: Optional[Union[str, bool]] = None,
+        default: Optional[str] = None,
+        _type: Optional[
+            Literal["bool", "int", "bool-or-int", "path", "expiry-date", "color"]
+        ] = None,
+        edit: Optional[bool] = None,
+        no_includes: Optional[bool] = None,
+        includes: Optional[bool] = None,
+        add: Optional[bool] = None,
+        **kwargs,
+    ):
+        """Status of working tree. Wraps
+        `git status <https://git-scm.com/docs/git-status>`_.
+
+        `git ls-files` has similar params (e.g. `z`)
+
+        Parameters
+        ----------
+        replace_all : Optional[bool]
+        get : Optional[bool]
+        get_all : Optional[bool]
+        get_regexp : Optional[bool]
+        get_urlmatch : Optional[tuple[str, str]]
+        system : Optional[bool]
+        local : Optional[bool]
+        worktree : Optional[bool]
+        file : Optional[StrOrBytesPath]
+        blob : Optional[str]
+        remove_section : Optional[bool]
+        rename_section : Optional[bool]
+        unset : Optional[bool]
+        unset_all : Optional[bool]
+        _list : Optional[bool]
+        fixed_value : Optional[bool]
+        no_type : Optional[bool]
+        null : Optional[bool]
+        name_only : Optional[bool]
+        show_origin : Optional[bool]
+        show_scope : Optional[bool]
+        get_color : Optional[Union[str, bool]]
+        get_colorbool : Optional[Union[str, bool]]
+        default : Optional[str]
+        _type : "bool", "int", "bool-or-int", "path", "expiry-date", "color"
+        edit : Optional[bool]
+        no_includes : Optional[bool]
+        includes : Optional[bool]
+        add : Optional[bool]
+
+        Examples
+        --------
+        >>> git = Git(dir=git_local_clone.dir)
+
+        >>> git.config()
+        'usage: git config ...'
+
+        >>> git.config(_list=True)
+        '...user.email=...'
+
+        >>> git.config(get='color.diff')
+        'auto'
+        """
+        local_flags: list[str] = []
+
+        if replace_all is True:
+            local_flags.append("--replace-all")
+
+        if get is not None and isinstance(get, str):
+            local_flags.extend(["--get", get])
+
+        if get_regexp is not None and isinstance(get_regexp, str):
+            local_flags.extend(["--get-regexp", get_regexp])
+
+        if get_all is not None and isinstance(get_all, str):
+            local_flags.extend(["--get-all", get_all])
+
+        if get_urlmatch is not None and isinstance(get_urlmatch, tuple):
+            local_flags.extend(["--get-urlmatch=", *get_urlmatch])
+
+        if unset is not None and isinstance(unset, str):
+            local_flags.extend(["--unset", unset])
+
+        if unset_all is not None and isinstance(unset_all, str):
+            local_flags.extend(["--unset-all", unset_all])
+
+        if _list is True:
+            local_flags.append("--list")
+
+        if fixed_value is True:
+            local_flags.append("--fixed-value")
+
+        if no_type is True:
+            local_flags.append("--no-type")
+
+        if null is True:
+            local_flags.append("--null")
+
+        if name_only is True:
+            local_flags.append("--name-only")
+
+        if show_origin is True:
+            local_flags.append("--show-origin")
+
+        if show_scope is True:
+            local_flags.append("--show-scope")
+
+        if edit is True:
+            local_flags.append("--edit")
+
+        if system is True:
+            local_flags.append("--system")
+
+        if local is True:
+            local_flags.append("--local")
+
+        if worktree is True:
+            local_flags.append("--worktree")
+
+        if remove_section is True:
+            local_flags.append("--remove-section")
+
+        if rename_section is True:
+            local_flags.append("--rename-section")
+
+        if _type is not None and isinstance(_type, str):
+            local_flags.extend(["--type", _type])
+
+        if blob is not None and isinstance(blob, str):
+            local_flags.extend(["--blob", blob])
+
+        if file is not None:
+            local_flags.extend(["--file", str(file)])
+
+        if default is True:
+            local_flags.append("--default")
+
+        if includes is True:
+            local_flags.append("--includes")
+
+        if no_includes is True:
+            local_flags.append("--no-includes")
+
+        if add is True:
+            local_flags.append("--add")
+
+        if get_colorbool is not None:
+            if isinstance(get_colorbool, str):
+                local_flags.extend(["--get-colorbool", get_colorbool])
+            else:
+                local_flags.append("--get-colorbool")
+
+        if get_color is not None:
+            if isinstance(get_color, str):
+                local_flags.extend(["--get-color", get_color])
+            else:
+                local_flags.append("--get-color")
+
+        return self.run(
+            ["config", *local_flags],
+            check_returncode=False,
+        )
